@@ -1,6 +1,7 @@
 from procrastinate import metadata as _metadata_module
 from procrastinate.aiopg_connector import AiopgConnector
 from procrastinate.app import App
+from procrastinate.blueprints import Blueprint
 from procrastinate.connector import BaseConnector
 from procrastinate.job_context import JobContext
 from procrastinate.psycopg2_connector import Psycopg2Connector
@@ -8,6 +9,7 @@ from procrastinate.retry import BaseRetryStrategy, RetryStrategy
 
 __all__ = [
     "App",
+    "Blueprint",
     "JobContext",
     "BaseConnector",
     "BaseRetryStrategy",
