@@ -71,7 +71,7 @@ def wrap_query_exceptions(coro: CoroutineFunction) -> CoroutineFunction:
                     continue
                 raise exc
         raise exceptions.ConnectorException(
-            "Could not get a valid connection after {} tries".format(max_tries)
+            f"Could not get a valid connection after {max_tries} tries"
         ) from final_exc
 
     return wrapped
@@ -227,9 +227,22 @@ class AiopgConnector(connector.BaseAsyncConnector):
     # Because of this, it's easier to have 2 distinct methods for executing from
     # a pool or from a connection
 
+    def _prepare_for_interpolation(self, query, has_arguments):
+        # psycopg2 thinks ``%`` are for it to process. If we have ``%`` in our query,
+        # like in RAISE, we need it to be passed to the database as-is, which means
+        # we need to escape the % by doubling it.
+        return (
+            query
+            if has_arguments or not isinstance(query, str)
+            else query.replace("%", "%%")
+        )
+
     @wrap_exceptions
     @wrap_query_exceptions
     async def execute_query_async(self, query: str, **arguments: Any) -> None:
+        query = self._prepare_for_interpolation(
+            query=query, has_arguments=bool(arguments)
+        )
         with await self.pool.cursor() as cursor:
             await cursor.execute(query, self._wrap_json(arguments))
 
@@ -238,6 +251,9 @@ class AiopgConnector(connector.BaseAsyncConnector):
     async def _execute_query_connection(
         self, query: str, connection: aiopg.Connection, **arguments: Any
     ) -> None:
+        query = self._prepare_for_interpolation(
+            query=query, has_arguments=bool(arguments)
+        )
         async with connection.cursor() as cursor:
             await cursor.execute(query, self._wrap_json(arguments))
 
@@ -246,6 +262,9 @@ class AiopgConnector(connector.BaseAsyncConnector):
     async def execute_query_one_async(
         self, query: str, **arguments: Any
     ) -> Dict[str, Any]:
+        query = self._prepare_for_interpolation(
+            query=query, has_arguments=bool(arguments)
+        )
         with await self.pool.cursor() as cursor:
             await cursor.execute(query, self._wrap_json(arguments))
 
@@ -256,6 +275,9 @@ class AiopgConnector(connector.BaseAsyncConnector):
     async def execute_query_all_async(
         self, query: str, **arguments: Any
     ) -> List[Dict[str, Any]]:
+        query = self._prepare_for_interpolation(
+            query=query, has_arguments=bool(arguments)
+        )
         with await self.pool.cursor() as cursor:
             await cursor.execute(query, self._wrap_json(arguments))
 
@@ -305,15 +327,19 @@ class AiopgConnector(connector.BaseAsyncConnector):
     ) -> None:
         # We'll leave this loop with a CancelledError, when we get cancelled
         while True:
-            # because of https://github.com/aio-libs/aiopg/issues/249,
-            # we could get stuck in here forever if the connection closes.
-            # That's why we need timeout and if connection is closed, reopen
-            # a new one.
+            # because of https://github.com/aio-libs/aiopg/issues/249 for
+            # aiopg<1.3.0, we could get stuck in here forever if the connection
+            # closes. That's why we need timeout and if connection is closed,
+            # reopen a new one.
             if connection.closed:
                 return
             try:
                 await asyncio.wait_for(connection.notifies.get(), timeout)
             except asyncio.TimeoutError:
                 continue
+            except psycopg2.Error:
+                # aiopg>=1.3.1 will raise if the connection is closed while
+                # we wait
+                continue
 
             event.set()
