@@ -1,20 +1,18 @@
 import functools
 import logging
-from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, List, Optional, Set
+from typing import TYPE_CHECKING, Any, Dict, Iterable, Optional, Union
 
-from procrastinate import admin
+from procrastinate import blueprints
 from procrastinate import connector as connector_module
-from procrastinate import exceptions, healthchecks, jobs
-from procrastinate import retry as retry_module
-from procrastinate import schema, store, utils
+from procrastinate import exceptions, jobs, manager, schema, utils
 
 if TYPE_CHECKING:
-    from procrastinate import tasks, worker
+    from procrastinate import worker
 
 logger = logging.getLogger(__name__)
 
 
-class App:
+class App(blueprints.Blueprint):
     """
     The App is the main entry point for procrastinate integration.
 
@@ -28,11 +26,8 @@ class App:
     tasks : ``Dict[str, tasks.Task]``
         The mapping of all tasks known by the app. Only procrastinate is expected to
         make changes to this mapping.
-    builtin_tasks : ``Dict[str, tasks.Task]``
-        The mapping of builtin tasks. Use it to programmatically access builtin tasks,
-        to defer them.
-    admin : ``admin.Admin``
-        The administration interface linked to the application.
+    job_manager : `manager.JobManager`
+        The `JobManager` linked to the application
     """
 
     @classmethod
@@ -64,16 +59,13 @@ class App:
             Typically an `AiopgConnector`. It will be responsible for all communications
             with the database. Mandatory.
         import_paths :
-            List of python dotted paths of modules to import, to make sure
-            that the workers know about all possible tasks.
-            If you fail to add a path here and a worker encounters
-            a task defined at that path, the task will be loaded on the
-            fly and run, but you will get a warning.
-            You don't need to specify paths that you know have already
-            been imported, though it doesn't hurt.
-            A `App.task` that has a custom "name" parameter, that is not
-            imported and whose module path is not in this list will
-            fail to run.
+            List of python dotted paths of modules to import, to make sure that
+            the workers know about all possible tasks. If there are tasks in a
+            module that is neither imported as a side effect of importing the
+            App, nor specified in this list, and a worker encounters a task
+            defined in that module, the task will fail (`TaskNotFound`). While
+            it is not mandatory to specify paths to modules that you know have
+            already been imported, it's a good idea to do so.
         worker_defaults :
             All the values passed here will override the default values sent when
             launching a worker. See `App.run_worker` for details.
@@ -89,108 +81,40 @@ class App:
         """
         from procrastinate import periodic
 
+        super().__init__()
+
         self.connector = connector
-        self.tasks: Dict[str, "tasks.Task"] = {}
-        self.builtin_tasks: Dict[str, "tasks.Task"] = {}
-        self.queues: Set[str] = set()
         self.import_paths = import_paths or []
         self.worker_defaults = worker_defaults or {}
         periodic_defaults = periodic_defaults or {}
 
-        self.job_store = store.JobStore(connector=self.connector)
+        self.job_manager = manager.JobManager(connector=self.connector)
         self.periodic_deferrer = periodic.PeriodicDeferrer(
-            job_store=self.job_store, **periodic_defaults
+            job_manager=self.job_manager, **periodic_defaults
         )
 
         self._register_builtin_tasks()
 
-    def task(
-        self,
-        _func: Optional[Callable] = None,
-        *,
-        name: Optional[str] = None,
-        aliases: Optional[List[str]] = None,
-        retry: retry_module.RetryValue = False,
-        pass_context: bool = False,
-        queue: str = jobs.DEFAULT_QUEUE,
-        lock: Optional[str] = None,
-        queueing_lock: Optional[str] = None,
-    ) -> Any:
+    def with_connector(self, connector: connector_module.BaseConnector) -> "App":
         """
-        Declare a function as a task. This method is meant to be used as a decorator::
+        Create another app instance sychronized with this one, with a different
+        connector. For all things regarding periodic tasks, the original app
+        (and its original connector) will be used, even when the new app's
+        methods are used.
 
-            @app.task(...)
-            def my_task(args):
-                ...
-
-        or::
-
-            @app.task
-            def my_task(args):
-                ...
-
-        The second form will use the default value for all parameters.
-
-        Parameters
-        ----------
-        _func :
-            The decorated function
-        queue :
-            The name of the queue in which jobs from this task will be launched, if
-            the queue is not overridden at launch.
-            Default is ``"default"``.
-            When a worker is launched, it can listen to specific queues, or to all
-            queues.
-        lock :
-            Default value for the ``lock`` (see `Task.defer`).
-        queueing_lock:
-            Default value for the ``queueing_lock`` (see `Task.defer`).
-        name :
-            Name of the task, by default the full dotted path to the decorated function.
-            if the function is nested or dynamically defined, it is important to give
-            it a unique name, and to make sure the module that defines this function
-            is listed in the ``import_paths`` of the `procrastinate.App`.
-        aliases:
-            Additional names for the task.
-            The main use case is to gracefully rename tasks by moving the old
-            name to aliases (these tasks can have been scheduled in a distant
-            future, so the aliases can remain for a long time).
-        retry :
-            Details how to auto-retry the task if it fails. Can be:
-
-            - A ``boolean``: will either not retry or retry indefinitely
-            - An ``int``: the number of retries before it gives up
-            - A `procrastinate.RetryStrategy` instance for complex cases
-
-            Default is no retry.
-        pass_context :
-            Passes the task execution context in the task as first
+        Returns
+        -------
+        `App`
+            A new compatible app.
         """
-        # Because of https://github.com/python/mypy/issues/3157, this function
-        # is quite impossible to type consistently, so, we're just using "Any"
-
-        def _wrap(func: Callable[..., "tasks.Task"]):
-            from procrastinate import tasks
-
-            task = tasks.Task(
-                func,
-                app=self,
-                queue=queue,
-                lock=lock,
-                queueing_lock=queueing_lock,
-                name=name,
-                aliases=aliases,
-                retry=retry,
-                pass_context=pass_context,
-            )
-            self._register(task)
-
-            return functools.update_wrapper(task, func)
-
-        if _func is None:  # Called as @app.task(...)
-            return _wrap
-
-        return _wrap(_func)  # Called as @app.task
+        app = App(
+            connector=connector,
+            import_paths=self.import_paths,
+            worker_defaults=self.worker_defaults,
+        )
+        app.tasks = self.tasks
+        app.periodic_deferrer = self.periodic_deferrer
+        return app
 
     def periodic(self, *, cron: str):
         """
@@ -204,22 +128,17 @@ class App:
         """
         return self.periodic_deferrer.periodic_decorator(cron=cron)
 
-    def _register(self, task: "tasks.Task") -> None:
-        self.tasks[task.name] = task
-        for alias in task.aliases:
-            self.tasks[alias] = task
-        queue = task.queue
-        if queue not in self.queues:
-            logger.debug(
-                f"Registering queue {queue}",
-                extra={"action": "register_queue", "queue": queue},
-            )
-            self.queues.add(queue)
-
     def _register_builtin_tasks(self) -> None:
         from procrastinate import builtin_tasks
 
-        builtin_tasks.register_builtin_tasks(self)
+        self.add_tasks_from(builtin_tasks.builtin, namespace="builtin")
+
+        # New tasks will be "builtin:procrastinate.builtin_tasks.remove_old_jobs"
+        # but for compatibility, we can keep the old name around.
+        self.add_task_alias(
+            task=builtin_tasks.remove_old_jobs,
+            alias="procrastinate.builtin_tasks.remove_old_jobs",
+        )
 
     def configure_task(
         self, name: str, *, allow_unknown: bool = True, **kwargs: Any
@@ -249,7 +168,7 @@ class App:
         except KeyError as exc:
             if allow_unknown:
                 return tasks.configure_task(
-                    name=name, job_store=self.job_store, **kwargs
+                    name=name, job_manager=self.job_manager, **kwargs
                 )
             raise exceptions.TaskNotFound from exc
 
@@ -298,35 +217,42 @@ class App:
             logs (defaults to ``None`` which will result in the worker named
             ``worker``).
         timeout : ``float``
-            Indicates the maximum duration (in seconds) procrastinate
-            workers wait between each database job poll.
-            Raising this parameter can lower the rate of workers making queries to the
-            database for requesting jobs.
+            Indicates the maximum duration (in seconds) the worker waits between
+            each database job poll. Raising this parameter can lower the rate at which
+            the worker makes queries to the database for requesting jobs.
             (defaults to 5.0)
         listen_notify : ``bool``
-            If ``True``, worker will dedicate a connection from the pool to listening to
-            database events, notifying of newly available jobs. If ``False``, workers
-            will just poll the database periodically (see ``timeout``). (defaults to
-            True)
+            If ``True``, the worker will dedicate a connection from the pool to
+            listening to database events, notifying of newly available jobs.
+            If ``False``, the worker will just poll the database periodically
+            (see ``timeout``). (defaults to ``True``)
+        delete_jobs : ``str``
+            If ``always``, the worker will automatically delete all jobs on completion.
+            If ``successful`` the worker will only delete successful jobs.
+            If ``never``, the worker will keep the jobs in the database.
+            (defaults to ``never``)
+        additional_context: ``Optional[Dict[str, Any]]``
+            If set extend the context received by the tasks when ``pass_context`` is set
+            to ``True`` in the task definition.
         """
         self.perform_import_paths()
         worker = self._worker(**kwargs)
         await worker.run()
 
+    async def check_connection_async(self):
+        return await self.job_manager.check_connection()
+
     @property
     def schema_manager(self) -> schema.SchemaManager:
         return schema.SchemaManager(connector=self.connector)
 
-    @property
-    def health_check_runner(self) -> healthchecks.HealthCheckRunner:
-        return healthchecks.HealthCheckRunner(connector=self.connector)
-
-    @property
-    def admin(self) -> admin.Admin:
-        return admin.Admin(connector=self.connector)
-
-    def open(self, pool: Optional[connector_module.Pool] = None) -> "App":
-        self.connector.open(pool)
+    def open(
+        self,
+        pool_or_engine: Optional[
+            Union[connector_module.Pool, connector_module.Engine]
+        ] = None,
+    ) -> "App":
+        self.connector.open(pool_or_engine)
         return self
 
     def close(self) -> None:
@@ -353,3 +279,4 @@ class App:
 
 
 utils.add_method_sync_api(cls=App, method_name="run_worker_async")
+utils.add_method_sync_api(cls=App, method_name="check_connection_async")
