@@ -30,14 +30,9 @@ def get_log_level(verbosity: int) -> int:
     return {0: logging.INFO, 1: logging.DEBUG}.get(min((1, verbosity)), 0)
 
 
-def click_set_verbosity(ctx: click.Context, param: click.Parameter, value: int) -> int:
-    set_verbosity(verbosity=value)
-    return value
-
-
-def set_verbosity(verbosity: int) -> None:
+def configure_logging(verbosity: int, format: str, style: str) -> None:
     level = get_log_level(verbosity=verbosity)
-    logging.basicConfig(level=level)
+    logging.basicConfig(level=level, format=format, style=style)
     level_name = logging.getLevelName(level)
     logger.debug(
         f"Log level set to {level_name}",
@@ -90,16 +85,27 @@ class MissingAppConnector(connector.BaseConnector):
 @click.option(
     "-v",
     "--verbose",
-    is_eager=True,
-    callback=click_set_verbosity,
     count=True,
     help="Use multiple times to increase verbosity",
 )
+@click.option(
+    "--log-format",
+    default=logging.BASIC_FORMAT,
+    help="Defines the format used for logging (see "
+    "https://docs.python.org/3/library/logging.html#logrecord-attributes)",
+)
+@click.option(
+    "--log-format-style",
+    default="%",
+    type=click.Choice("%{$"),
+    help="Defines the style for the log format string (see "
+    "https://docs.python.org/3/howto/logging-cookbook.html#use-of-alternative-formatting-styles)",
+)
 @click.version_option(
     procrastinate.__version__, "-V", "--version", prog_name=PROGRAM_NAME
 )
 @handle_errors()
-def cli(ctx: click.Context, app: str, **kwargs) -> None:
+def cli(ctx: click.Context, app: str, verbose, log_format, log_format_style) -> None:
     """
     Interact with a Procrastinate app. See subcommands for details.
 
@@ -107,6 +113,7 @@ def cli(ctx: click.Context, app: str, **kwargs) -> None:
     or PROCRASTINATE_COMMAND_UPPERCASE_NAME (examples: PROCRASTINATE_APP,
     PROCRASTINATE_DEFER_UNKNOWN, ...).
     """
+    configure_logging(verbosity=verbose, format=log_format, style=log_format_style)
     if app:
         app_obj = procrastinate.App.from_path(dotted_path=app)
     else:
@@ -121,7 +128,11 @@ def cli(ctx: click.Context, app: str, **kwargs) -> None:
     ctx.default_map = {"worker": worker_defaults}
 
 
-@cli.resultcallback()
+# result_callback for click >=8.0, resultcallback for click <8.1
+result_callback = getattr(cli, "result_callback") or cli.resultcallback
+
+
+@result_callback()
 @click.pass_obj
 def close_connection(procrastinate_app: procrastinate.App, *args, **kwargs):
     # There's an internal click param named app, we can't name our variable "app" too.
@@ -167,6 +178,12 @@ def close_connection(procrastinate_app: procrastinate.App, *args, **kwargs):
     default=True,
     help="Whether to actively listen for new jobs or periodically poll",
 )
+@click.option(
+    "--delete-jobs",
+    type=click.Choice([v.value for v in worker.DeleteJobCondition]),
+    default=worker.DeleteJobCondition.NEVER.value,
+    help="Whether to delete jobs on completion",
+)
 @handle_errors()
 def worker_(app: procrastinate.App, queues: str, **kwargs):
     """
@@ -362,16 +379,19 @@ def healthchecks(app: procrastinate.App):
     """
     Check the state of procrastinate.
     """
-    health_check = app.health_check_runner
-    db_ok = health_check.check_connection()  # type: ignore
-    if not db_ok:
-        click.echo("Cannot connect to DB")
-        return  # No need to go further
+    db_ok = app.check_connection()  # type: ignore
+    # If app or DB is not configured correctly, we raise before this point
+    click.echo("App configuration: OK")
     click.echo("DB connection: OK")
 
-    status_count = health_check.get_status_count()  # type: ignore
-    for status, count in status_count.items():
-        click.echo(f"{status.value}: {count}")
+    if not db_ok:
+        raise click.ClickException(
+            "Connection to the database works but the procrastinate_jobs table was not "
+            "found. Have you applied database migrations (see "
+            "`procrastinate schema -h`)?"
+        )
+
+    click.echo("Found procrastinate_jobs table: OK")
 
 
 @cli.command("shell")
@@ -381,7 +401,7 @@ def shell_(app: procrastinate.App):
     """
     Administration shell for procrastinate.
     """
-    shell.ProcrastinateShell(app.admin).cmdloop()
+    shell.ProcrastinateShell(job_manager=app.job_manager).cmdloop()
 
 
 def main():
