@@ -3,10 +3,8 @@ from typing import Any, Callable, Dict, Iterable, List, Optional
 
 from procrastinate import exceptions, utils
 
-QUEUEING_LOCK_CONSTRAINT = "procrastinate_jobs_queueing_lock_idx"
-
-
-Pool = Any  # The connection pool can be any pool object compatible with the database.
+Pool = Any
+Engine = Any
 
 
 class BaseConnector:
@@ -53,7 +51,10 @@ class BaseConnector:
         raise exceptions.SyncConnectorConfigurationError
 
     def __del__(self):
-        self.close()
+        try:
+            self.close()
+        except NotImplementedError:
+            pass
 
 
 @utils.add_sync_api
