@@ -1,6 +1,11 @@
+import sys
 from typing import Mapping
 
-import importlib_metadata
+# https://github.com/pypa/twine/pull/551
+if sys.version_info[:2] < (3, 8):  # coverage: exclude
+    import importlib_metadata
+else:  # coverage: exclude
+    import importlib.metadata as importlib_metadata
 
 
 def extract_metadata() -> Mapping[str, str]:
