@@ -1,7 +1,8 @@
 import contextlib
 import functools
 import logging
-from typing import Any, Callable, Dict, Optional
+import re
+from typing import Any, Callable, Dict, Iterator, Optional
 
 import psycopg2
 import psycopg2.errors
@@ -15,8 +16,7 @@ logger = logging.getLogger(__name__)
 
 def wrap_exceptions(func: Callable) -> Callable:
     """
-    Wrap psycopg2 errors as connector exceptions
-    This decorator is expected to be used on coroutine functions only
+    Wrap psycopg2 errors as connector exceptions.
     """
 
     @functools.wraps(func)
@@ -60,12 +60,15 @@ def wrap_query_exceptions(func: Callable) -> Callable:
             except psycopg2.errors.AdminShutdown:
                 continue
         raise exceptions.ConnectorException(
-            "Could not get a valid connection after {} tries".format(max_tries)
+            f"Could not get a valid connection after {max_tries} tries"
         ) from final_exc
 
     return wrapped
 
 
+PERCENT_PATTERN = re.compile(r"%(?![\(s])")
+
+
 class Psycopg2Connector(connector.BaseConnector):
     @wrap_exceptions
     def __init__(
@@ -73,7 +76,6 @@ class Psycopg2Connector(connector.BaseConnector):
         *,
         json_dumps: Optional[Callable] = None,
         json_loads: Optional[Callable] = None,
-        pool: Optional[psycopg2.pool.ThreadedConnectionPool] = None,
         **kwargs: Any,
     ):
         """
@@ -179,7 +181,7 @@ class Psycopg2Connector(connector.BaseConnector):
         }
 
     @contextlib.contextmanager
-    def _connection(self) -> psycopg2.extensions.connection:
+    def _connection(self) -> Iterator[psycopg2.extensions.connection]:
         # in case of an admin shutdown (Postgres error code 57P01) we do not
         # rollback the connection or put the connection back to the pool as
         # this will cause a psycopg2.InterfaceError exception
@@ -201,14 +203,18 @@ class Psycopg2Connector(connector.BaseConnector):
     def execute_query(self, query: str, **arguments: Any) -> None:
         with self._connection() as connection:
             with connection.cursor() as cursor:
-                cursor.execute(query, self._wrap_json(arguments))
+                cursor.execute(
+                    PERCENT_PATTERN.sub("%%", query), self._wrap_json(arguments)
+                )
 
     @wrap_exceptions
     @wrap_query_exceptions
     def execute_query_one(self, query: str, **arguments: Any) -> Dict[str, Any]:
         with self._connection() as connection:
             with connection.cursor() as cursor:
-                cursor.execute(query, self._wrap_json(arguments))
+                cursor.execute(
+                    PERCENT_PATTERN.sub("%%", query), self._wrap_json(arguments)
+                )
                 return cursor.fetchone()
 
     @wrap_exceptions
@@ -216,5 +222,7 @@ class Psycopg2Connector(connector.BaseConnector):
     def execute_query_all(self, query: str, **arguments: Any) -> Dict[str, Any]:
         with self._connection() as connection:
             with connection.cursor() as cursor:
-                cursor.execute(query, self._wrap_json(arguments))
+                cursor.execute(
+                    PERCENT_PATTERN.sub("%%", query), self._wrap_json(arguments)
+                )
                 return cursor.fetchall()
