@@ -1,7 +1,12 @@
 import re
+import sys
 from typing import Dict
 
-from importlib_resources import read_text
+# https://github.com/pypa/twine/pull/551
+if sys.version_info[:2] < (3, 9):  # coverage: exclude
+    import importlib_resources
+else:  # coverage: exclude
+    import importlib.resources as importlib_resources
 
 QUERIES_REGEX = re.compile(r"(?:\n|^)-- ([a-z0-9_]+) --\n(?:-- .+\n)*", re.MULTILINE)
 
@@ -21,7 +26,9 @@ def parse_query_file(query_file: str) -> Dict["str", "str"]:
 
 
 def get_queries() -> Dict["str", "str"]:
-    return parse_query_file(read_text("procrastinate.sql", "queries.sql"))
+    return parse_query_file(
+        importlib_resources.read_text("procrastinate.sql", "queries.sql")
+    )
 
 
 queries = get_queries()
