@@ -14,12 +14,12 @@ SELECT procrastinate_defer_periodic_job(%(queue)s, %(lock)s, %(queueing_lock)s,
 
 -- fetch_job --
 -- Get the first awaiting job
-SELECT id, task_name, lock, queueing_lock, args, scheduled_at, queue_name, attempts
+SELECT id, status, task_name, lock, queueing_lock, args, scheduled_at, queue_name, attempts
     FROM procrastinate_fetch_job(%(queues)s);
 
 -- select_stalled_jobs --
 -- Get running jobs that started more than a given time ago
-SELECT job.id, task_name, lock, queueing_lock, args, scheduled_at, queue_name, attempts, max(event.at) started_at
+SELECT job.id, status, task_name, lock, queueing_lock, args, scheduled_at, queue_name, attempts, max(event.at) started_at
     FROM procrastinate_jobs job
     JOIN procrastinate_events event
       ON event.job_id = job.id
@@ -47,16 +47,20 @@ WHERE id IN (
 )
 
 -- finish_job --
--- Stop a job, free the lock and record the relevant events
-SELECT procrastinate_finish_job(%(job_id)s, %(status)s, %(scheduled_at)s);
+-- Finish a job, changing it from "doing" to "succeeded" or "failed"
+SELECT procrastinate_finish_job(%(job_id)s, %(status)s, NULL, %(delete_job)s);
+
+-- retry_job --
+-- Retry a job, changing it from "doing" to "todo"
+SELECT procrastinate_retry_job(%(job_id)s, %(retry_at)s);
 
 -- listen_queue --
 -- In this one, the argument is an identifier, shoud not be escaped the same way
 LISTEN {channel_name};
 
 -- check_connection --
--- This does only check you have read permission
-SELECT TRUE as check;
+-- This query checks that the procrastinate_jobs table exists. Returns NULL otherwise.
+SELECT to_regclass('procrastinate_jobs') as check;
 
 -- count_jobs_status --
 -- Count the number of jobs per status
@@ -146,7 +150,31 @@ SELECT task_name AS name,
  GROUP BY name
  ORDER BY name;
 
--- set_job_status --
-UPDATE procrastinate_jobs
-   SET status = %(status)s
- WHERE id = %(id)s
+-- list_locks --
+-- Get list of locks and number of jobs per lock
+WITH jobs AS (
+  SELECT
+    id,
+    lock,
+    status
+  FROM procrastinate_jobs
+  WHERE (%(queue_name)s IS NULL OR queue_name = %(queue_name)s)
+  AND (%(task_name)s IS NULL OR task_name = %(task_name)s)
+  AND (%(status)s IS NULL OR status = %(status)s)
+  AND (%(lock)s IS NULL OR lock = %(lock)s)
+  AND lock IS NOT NULL
+), locks AS (
+  SELECT
+    lock,
+    status,
+    count(id) AS jobs_count
+  FROM jobs
+  GROUP BY lock, status
+)
+SELECT
+  lock AS name,
+  sum(jobs_count) AS jobs_count,
+  json_object_agg(status, jobs_count) AS stats
+FROM locks
+GROUP BY name
+ORDER BY name;
