@@ -59,7 +59,10 @@ CREATE UNIQUE INDEX procrastinate_jobs_queueing_lock_idx ON procrastinate_jobs (
 CREATE UNIQUE INDEX procrastinate_jobs_lock_idx ON procrastinate_jobs (lock) WHERE status = 'doing';
 
 CREATE INDEX procrastinate_jobs_queue_name_idx ON procrastinate_jobs(queue_name);
+CREATE INDEX procrastinate_jobs_id_lock_idx ON procrastinate_jobs (id, lock) WHERE status = ANY (ARRAY['todo'::procrastinate_job_status, 'doing'::procrastinate_job_status]);
+
 CREATE INDEX procrastinate_events_job_id_fkey ON procrastinate_events(job_id);
+
 CREATE INDEX procrastinate_periodic_defers_job_id_fkey ON procrastinate_periodic_defers(job_id);
 
 
@@ -74,7 +77,7 @@ CREATE FUNCTION procrastinate_defer_job(
     scheduled_at timestamp with time zone
 ) RETURNS bigint
     LANGUAGE plpgsql
-    AS $$
+AS $$
 DECLARE
 	job_id bigint;
 BEGIN
@@ -94,7 +97,7 @@ CREATE FUNCTION procrastinate_defer_periodic_job(
     _defer_timestamp bigint
 ) RETURNS bigint
     LANGUAGE plpgsql
-    AS $$
+AS $$
 DECLARE
 	_job_id bigint;
 	_defer_id bigint;
@@ -141,7 +144,7 @@ $$;
 
 CREATE FUNCTION procrastinate_fetch_job(target_queue_names character varying[]) RETURNS procrastinate_jobs
     LANGUAGE plpgsql
-    AS $$
+AS $$
 DECLARE
 	found_jobs procrastinate_jobs;
 BEGIN
@@ -153,7 +156,9 @@ BEGIN
                 NOT EXISTS (
                     SELECT 1
                         FROM procrastinate_jobs AS earlier_jobs
-                        WHERE earlier_jobs.lock = jobs.lock
+                        WHERE
+                            jobs.lock IS NOT NULL
+                            AND earlier_jobs.lock = jobs.lock
                             AND earlier_jobs.status IN ('todo', 'doing')
                             AND earlier_jobs.id < jobs.id)
                 AND jobs.status = 'todo'
@@ -172,9 +177,11 @@ BEGIN
 END;
 $$;
 
+-- procrastinate_finish_job – old version
+-- to remove after 1.0.0 is released
 CREATE FUNCTION procrastinate_finish_job(job_id integer, end_status procrastinate_job_status, next_scheduled_at timestamp with time zone) RETURNS void
     LANGUAGE plpgsql
-    AS $$
+AS $$
 BEGIN
     UPDATE procrastinate_jobs
     SET status = end_status,
@@ -184,9 +191,73 @@ BEGIN
 END;
 $$;
 
+-- procrastinate_finish_job – old version
+-- to remove after 1.0.0 is released
+CREATE FUNCTION procrastinate_finish_job(job_id integer, end_status procrastinate_job_status) RETURNS void
+    LANGUAGE plpgsql
+AS $$
+BEGIN
+    UPDATE procrastinate_jobs
+    SET status = end_status,
+        attempts = attempts + 1
+    WHERE id = job_id;
+END;
+$$;
+
+-- procrastinate_finish_job
+-- the next_scheduled_at argument is kept for compatibility reasons, it is to be
+-- removed after 1.0.0 is released
+CREATE FUNCTION procrastinate_finish_job(job_id integer, end_status procrastinate_job_status, next_scheduled_at timestamp with time zone, delete_job boolean) RETURNS void
+    LANGUAGE plpgsql
+AS $$
+DECLARE
+    _job_id bigint;
+BEGIN
+    IF end_status NOT IN ('succeeded', 'failed') THEN
+        RAISE 'End status should be either "succeeded" or "failed" (job id: %)', job_id;
+    END IF;
+    IF delete_job THEN
+        DELETE FROM procrastinate_jobs
+        WHERE id = job_id AND status IN ('todo', 'doing')
+        RETURNING id INTO _job_id;
+    ELSE
+        UPDATE procrastinate_jobs
+        SET status = end_status,
+            attempts =
+                CASE
+                    WHEN status = 'doing' THEN attempts + 1
+                    ELSE attempts
+                END
+        WHERE id = job_id AND status IN ('todo', 'doing')
+        RETURNING id INTO _job_id;
+    END IF;
+    IF _job_id IS NULL THEN
+        RAISE 'Job was not found or not in "doing" or "todo" status (job id: %)', job_id;
+    END IF;
+END;
+$$;
+
+CREATE FUNCTION procrastinate_retry_job(job_id integer, retry_at timestamp with time zone) RETURNS void
+    LANGUAGE plpgsql
+AS $$
+DECLARE
+    _job_id bigint;
+BEGIN
+    UPDATE procrastinate_jobs
+    SET status = 'todo',
+        attempts = attempts + 1,
+        scheduled_at = retry_at
+    WHERE id = job_id AND status = 'doing'
+    RETURNING id INTO _job_id;
+    IF _job_id IS NULL THEN
+        RAISE 'Job was not found or not in "doing" status (job id: %)', job_id;
+    END IF;
+END;
+$$;
+
 CREATE FUNCTION procrastinate_notify_queue() RETURNS trigger
     LANGUAGE plpgsql
-    AS $$
+AS $$
 BEGIN
 	PERFORM pg_notify('procrastinate_queue#' || NEW.queue_name, NEW.task_name);
 	PERFORM pg_notify('procrastinate_any_queue', NEW.task_name);
@@ -196,7 +267,7 @@ $$;
 
 CREATE FUNCTION procrastinate_trigger_status_events_procedure_insert() RETURNS trigger
     LANGUAGE plpgsql
-    AS $$
+AS $$
 BEGIN
     INSERT INTO procrastinate_events(job_id, type)
         VALUES (NEW.id, 'deferred'::procrastinate_job_event_type);
@@ -206,7 +277,7 @@ $$;
 
 CREATE FUNCTION procrastinate_trigger_status_events_procedure_update() RETURNS trigger
     LANGUAGE plpgsql
-    AS $$
+AS $$
 BEGIN
     WITH t AS (
         SELECT CASE
@@ -241,7 +312,7 @@ $$;
 
 CREATE FUNCTION procrastinate_trigger_scheduled_events_procedure() RETURNS trigger
     LANGUAGE plpgsql
-    AS $$
+AS $$
 BEGIN
     INSERT INTO procrastinate_events(job_id, type, at)
         VALUES (NEW.id, 'scheduled'::procrastinate_job_event_type, NEW.scheduled_at);
@@ -250,6 +321,17 @@ BEGIN
 END;
 $$;
 
+CREATE FUNCTION procrastinate_unlink_periodic_defers() RETURNS trigger
+    LANGUAGE plpgsql
+AS $$
+BEGIN
+    UPDATE procrastinate_periodic_defers
+    SET job_id = NULL
+    WHERE job_id = OLD.id;
+    RETURN OLD;
+END;
+$$;
+
 -- Triggers
 
 CREATE TRIGGER procrastinate_jobs_notify_queue
@@ -271,3 +353,7 @@ CREATE TRIGGER procrastinate_trigger_scheduled_events
     AFTER UPDATE OR INSERT ON procrastinate_jobs
     FOR EACH ROW WHEN ((new.scheduled_at IS NOT NULL AND new.status = 'todo'::procrastinate_job_status))
     EXECUTE PROCEDURE procrastinate_trigger_scheduled_events_procedure();
+
+CREATE TRIGGER procrastinate_trigger_delete_jobs
+    BEFORE DELETE ON procrastinate_jobs
+    FOR EACH ROW EXECUTE PROCEDURE procrastinate_unlink_periodic_defers();
