@@ -2,26 +2,18 @@ import datetime
 import logging
 from typing import Any, Callable, Dict, List, Optional
 
-from procrastinate import app, exceptions, jobs
+from procrastinate import app as app_module
+from procrastinate import blueprints, exceptions, jobs, manager
 from procrastinate import retry as retry_module
-from procrastinate import store, types, utils
+from procrastinate import types, utils
 
 logger = logging.getLogger(__name__)
 
 
-def load_task(path: str) -> "Task":
-    try:
-        task = utils.load_from_path(path, Task)
-    except exceptions.LoadFromPathError as exc:
-        raise exceptions.TaskNotFound(f"Task at {path} cannot be imported: {str(exc)}")
-
-    return task
-
-
 def configure_task(
     *,
     name: str,
-    job_store: store.JobStore,
+    job_manager: manager.JobManager,
     lock: Optional[str] = None,
     queueing_lock: Optional[str] = None,
     task_kwargs: Optional[types.JSONDict] = None,
@@ -46,7 +38,7 @@ def configure_task(
             task_kwargs=task_kwargs,
             scheduled_at=schedule_at,
         ),
-        job_store=job_store,
+        job_manager=job_manager,
     )
 
 
@@ -81,7 +73,7 @@ class Task:
         self,
         func: Callable,
         *,
-        app: app.App,
+        blueprint: "blueprints.Blueprint",
         # task naming
         name: Optional[str] = None,
         aliases: Optional[List[str]] = None,
@@ -94,7 +86,7 @@ class Task:
         queueing_lock: Optional[str] = None,
     ):
         self.queue = queue
-        self.app = app
+        self.blueprint = blueprint
         self.func: Callable = func
         self.aliases = aliases if aliases else []
         self.retry_strategy = retry_module.get_retry_strategy(retry)
@@ -103,6 +95,16 @@ class Task:
         self.lock = lock
         self.queueing_lock = queueing_lock
 
+    def add_namespace(self, namespace: str) -> None:
+        """
+        Prefix the given namespace to the name and aliases of the task.
+        """
+        self.name = utils.add_namespace(name=self.name, namespace=namespace)
+        self.aliases = [
+            utils.add_namespace(name=alias, namespace=namespace)
+            for alias in self.aliases
+        ]
+
     def __call__(self, *args, **kwargs: types.JSONValue) -> Any:
         return self.func(*args, **kwargs)
 
@@ -176,9 +178,14 @@ class Task:
         ValueError
             If you try to define both schedule_at and schedule_in
         """
+        if not isinstance(self.blueprint, app_module.App):
+            raise exceptions.UnboundTaskError
+
+        app = self.blueprint
+
         return configure_task(
             name=self.name,
-            job_store=self.app.job_store,
+            job_manager=app.job_manager,
             lock=lock if lock is not None else self.lock,
             queueing_lock=(
                 queueing_lock if queueing_lock is not None else self.queueing_lock
