@@ -32,7 +32,8 @@ class InMemoryConnector(connector.BaseAsyncConnector):
 
     def reset(self):
         """
-        Removes anything the store contains, to ensure test independence.
+        Removes anything the in-memory pseudo-database contains, to ensure test
+        independence.
         """
         self.jobs: Dict[int, JobRow] = {}
         self.events: Dict[int, List[EventRow]] = {}
@@ -41,6 +42,7 @@ class InMemoryConnector(connector.BaseAsyncConnector):
         self.notify_event = None
         self.notify_channels = []
         self.periodic_defers: Dict[str, int] = {}
+        self.table_exists = True
 
     def generic_execute(self, query, suffix, **arguments) -> Any:
         """
@@ -91,11 +93,14 @@ class InMemoryConnector(connector.BaseAsyncConnector):
     def defer_job_one(
         self, task_name, lock, queueing_lock, args, scheduled_at, queue
     ) -> JobRow:
-        if any(
-            job for job in self.jobs.values() if job["queueing_lock"] == queueing_lock
+        if queueing_lock is not None and any(
+            job["queueing_lock"] == queueing_lock and job["status"] == "todo"
+            for job in self.jobs.values()
         ):
+            from . import manager
+
             raise exceptions.UniqueViolation(
-                constraint_name=connector.QUEUEING_LOCK_CONSTRAINT
+                constraint_name=manager.QUEUEING_LOCK_CONSTRAINT
             )
 
         id = next(self.job_counter)
@@ -139,7 +144,9 @@ class InMemoryConnector(connector.BaseAsyncConnector):
 
     @property
     def current_locks(self) -> Iterable[str]:
-        return {job["lock"] for job in self.jobs.values() if job["status"] == "doing"}
+        return {
+            job["lock"] for job in self.jobs.values() if job["status"] == "doing"
+        } - {None}
 
     @property
     def finished_jobs(self) -> List[JobRow]:
@@ -166,21 +173,23 @@ class InMemoryConnector(connector.BaseAsyncConnector):
 
         return {"id": None}
 
-    def finish_job_run(
-        self, job_id: int, status: str, scheduled_at: Optional[datetime.datetime] = None
-    ) -> None:
+    def finish_job_run(self, job_id: int, status: str, delete_job: bool) -> None:
+        if delete_job:
+            self.jobs.pop(job_id)
+            return
+
         job_row = self.jobs[job_id]
         job_row["status"] = status
-        event_type = status
+        job_row["attempts"] += 1
+        self.events[job_id].append({"type": status, "at": utils.utcnow()})
 
-        if status == "todo":
-            job_row["attempts"] += 1
-            job_row["scheduled_at"] = scheduled_at
-            if scheduled_at:
-                self.events[job_id].append({"type": "scheduled", "at": scheduled_at})
-            event_type = "deferred_for_retry"
-
-        self.events[job_id].append({"type": event_type, "at": utils.utcnow()})
+    def retry_job_run(self, job_id: int, retry_at: datetime.datetime) -> None:
+        job_row = self.jobs[job_id]
+        job_row["status"] = "todo"
+        job_row["attempts"] += 1
+        job_row["scheduled_at"] = retry_at
+        self.events[job_id].append({"type": "scheduled", "at": retry_at})
+        self.events[job_id].append({"type": "deferred_for_retry", "at": utils.utcnow()})
 
     def select_stalled_jobs_all(self, nb_seconds, queue, task_name):
         return (
@@ -236,6 +245,17 @@ class InMemoryConnector(connector.BaseAsyncConnector):
             stats = Counter(job["status"] for job in task_jobs)
             yield {"name": task, "jobs_count": len(task_jobs), "stats": stats}
 
+    def list_locks_all(self, **kwargs):
+        jobs = list(self.list_jobs_all(**kwargs))
+        locks = sorted({job["lock"] for job in jobs})
+        for lock in locks:
+            lock_jobs = [job for job in jobs if job["lock"] == lock]
+            stats = Counter(job["status"] for job in lock_jobs)
+            yield {"name": lock, "jobs_count": len(lock_jobs), "stats": stats}
+
     def set_job_status_run(self, id, status):
         id = int(id)
         self.jobs[id]["status"] = status
+
+    def check_connection_one(self):
+        return {"check": self.table_exists or None}
