@@ -1,8 +1,8 @@
 import asyncio
-import contextlib
 import logging
 import time
-from typing import Dict, Iterable, Optional, Set, Union
+from enum import Enum
+from typing import Any, Dict, Iterable, Optional, Union
 
 from procrastinate import app, exceptions, job_context, jobs, signals, tasks, utils
 
@@ -14,16 +14,28 @@ WORKER_TIMEOUT = 5.0  # seconds
 WORKER_CONCURRENCY = 1  # parallel task(s)
 
 
+class DeleteJobCondition(Enum):
+    """
+    An enumeration with all the possible conditions to delete a job
+    """
+
+    NEVER = "never"  #: Keep jobs in database after completion
+    SUCCESSFUL = "successful"  #: Delete only successful jobs
+    ALWAYS = "always"  #: Always delete jobs at completion
+
+
 class Worker:
     def __init__(
         self,
-        app: app.App,
+        app: "app.App",
         queues: Optional[Iterable[str]] = None,
         name: Optional[str] = None,
-        concurrency: int = 1,
+        concurrency: int = WORKER_CONCURRENCY,
         wait: bool = True,
         timeout: float = WORKER_TIMEOUT,
         listen_notify: bool = True,
+        delete_jobs: str = DeleteJobCondition.NEVER.value,
+        additional_context: Optional[Dict[str, Any]] = None,
     ):
         self.app = app
         self.queues = queues
@@ -33,18 +45,21 @@ class Worker:
         self.timeout = timeout
         self.wait = wait
         self.listen_notify = listen_notify
+        self.delete_jobs = DeleteJobCondition(delete_jobs)
 
-        # Handling the info about the currently running task.
-        self.known_missing_tasks: Set[str] = set()
-        self.job_store = self.app.job_store
+        self.job_manager = self.app.job_manager
 
         if name:
             self.logger = logger.getChild(name)
         else:
             self.logger = logger
 
+        # Handling the info about the currently running task.
         self.base_context: job_context.JobContext = job_context.JobContext(
-            app=app, worker_name=self.worker_name, worker_queues=self.queues
+            app=app,
+            worker_name=self.worker_name,
+            worker_queues=self.queues,
+            additional_context=additional_context.copy() if additional_context else {},
         )
         self.current_contexts: Dict[int, job_context.JobContext] = {}
         self.stop_requested = False
@@ -62,6 +77,7 @@ class Worker:
         if reset or worker_id not in self.current_contexts:
             context = self.base_context
             kwargs["worker_id"] = worker_id
+            kwargs["additional_context"] = self.base_context.additional_context.copy()
         else:
             context = self.current_contexts[worker_id]
 
@@ -71,19 +87,15 @@ class Worker:
 
         return context
 
-    def listener(self):
+    async def listener(self):
         assert self.notify_event
-        return utils.task_context(
-            awaitable=self.job_store.listen_for_jobs(
-                event=self.notify_event, queues=self.queues
-            ),
-            name="listener",
+        return await self.job_manager.listen_for_jobs(
+            event=self.notify_event,
+            queues=self.queues,
         )
 
-    def periodic_deferrer(self):
-        return utils.task_context(
-            awaitable=self.app.periodic_deferrer.worker(), name="periodic_deferrer"
-        )
+    async def periodic_deferrer(self):
+        return await self.app.periodic_deferrer.worker()
 
     async def run(self) -> None:
         self.notify_event = asyncio.Event()
@@ -96,18 +108,18 @@ class Worker:
             ),
         )
 
-        with contextlib.ExitStack() as stack:
+        with signals.on_stop(self.stop):
+            side_coros = [self.periodic_deferrer()]
             if self.wait and self.listen_notify:
-                stack.enter_context(self.listener())
+                side_coros.append(self.listener())
 
-            stack.enter_context(self.periodic_deferrer())
-            stack.enter_context(signals.on_stop(self.stop))
-
-            await asyncio.gather(
-                *(
+            await utils.run_tasks(
+                main_coros=(
                     self.single_worker(worker_id=worker_id)
                     for worker_id in range(self.concurrency)
-                )
+                ),
+                side_coros=side_coros,
+                graceful_stop_callback=self.stop,
             )
 
         self.logger.info(
@@ -119,7 +131,7 @@ class Worker:
     async def single_worker(self, worker_id: int):
         current_timeout = self.timeout * (worker_id + 1)
         while not self.stop_requested:
-            job = await self.job_store.fetch_job(self.queues)
+            job = await self.job_manager.fetch_job(self.queues)
             if job:
                 await self.process_job(job=job, worker_id=worker_id)
             else:
@@ -131,7 +143,7 @@ class Worker:
     async def wait_for_job(self, timeout: float):
         assert self.notify_event
         self.logger.debug(
-            f"Waiting for new jobs on queues " f"{self.base_context.queues_display}",
+            f"Waiting for new jobs on {self.base_context.queues_display}",
             extra=self.base_context.log_extra(
                 action="waiting_for_jobs", queues=self.queues
             ),
@@ -152,25 +164,35 @@ class Worker:
             extra=context.log_extra(action="loaded_job_info"),
         )
 
-        status = jobs.Status.FAILED
-        next_attempt_scheduled_at = None
+        status, retry_at = None, None
         try:
             await self.run_job(job=job, worker_id=worker_id)
             status = jobs.Status.SUCCEEDED
         except exceptions.JobRetry as e:
-            status = jobs.Status.TODO
-            next_attempt_scheduled_at = e.scheduled_at
+            retry_at = e.scheduled_at
         except exceptions.JobError:
-            pass
+            status = jobs.Status.FAILED
         except exceptions.TaskNotFound as exc:
+            status = jobs.Status.FAILED
             self.logger.exception(
                 f"Task was not found: {exc}",
                 extra=context.log_extra(action="task_not_found", exception=str(exc)),
             )
         finally:
-            await self.job_store.finish_job(
-                job=job, status=status, scheduled_at=next_attempt_scheduled_at
-            )
+            if retry_at:
+                await self.job_manager.retry_job(job=job, retry_at=retry_at)
+            else:
+                assert status is not None
+
+                delete_job = {
+                    DeleteJobCondition.ALWAYS: True,
+                    DeleteJobCondition.NEVER: False,
+                    DeleteJobCondition.SUCCESSFUL: status == jobs.Status.SUCCEEDED,
+                }[self.delete_jobs]
+
+                await self.job_manager.finish_job(
+                    job=job, status=status, delete_job=delete_job
+                )
 
             self.logger.debug(
                 f"Acknowledged job completion {job.call_string}",
@@ -179,41 +201,21 @@ class Worker:
             # Remove job information from the current context
             self.context_for_worker(worker_id=worker_id, reset=True)
 
-    def load_task(self, task_name: str, worker_id: int) -> tasks.Task:
-        if task_name in self.known_missing_tasks:
-            raise exceptions.TaskNotFound(f"Cancelling job for {task_name} (not found)")
-
+    def find_task(self, task_name: str) -> tasks.Task:
         try:
-            # Simple case: the task is already known
             return self.app.tasks[task_name]
         except KeyError:
-            pass
-
-        # Will raise if not found or not a task
-        try:
-            task = tasks.load_task(task_name)
-        except exceptions.ProcrastinateException:
-            self.known_missing_tasks.add(task_name)
-            raise
-
-        context = self.context_for_worker(worker_id=worker_id)
-
-        self.logger.warning(
-            f"Task at {task_name} was not registered, it's been loaded dynamically.",
-            extra=context.log_extra(action="load_dynamic_task", task_name=task_name),
-        )
-
-        self.app.tasks[task_name] = task
-        return task
+            raise exceptions.TaskNotFound
 
     async def run_job(self, job: jobs.Job, worker_id: int) -> None:
         task_name = job.task_name
 
-        task = self.load_task(task_name=task_name, worker_id=worker_id)
+        task = self.find_task(task_name=task_name)
 
         context = self.context_for_worker(worker_id=worker_id, task=task)
 
-        start_time = context.additional_context["start_timestamp"] = time.time()
+        start_time = time.time()
+        context.job_result.start_timestamp = start_time
 
         self.logger.info(
             f"Starting job {job.call_string}",
@@ -256,13 +258,8 @@ class Worker:
         finally:
             end_time = time.time()
             duration = end_time - start_time
-            context.additional_context.update(
-                {
-                    "end_timestamp": end_time,
-                    "duration": duration,
-                    "result": task_result,
-                }
-            )
+            context.job_result.end_timestamp = end_time
+            context.job_result.result = task_result
 
             extra = context.log_extra(action=log_action)
 
